import wtk_graph;

size(6cm,4cm,IgnoreAspect);

scale(Linear,Linear);

real a = 0.2;
real b = 0.7;
real F = 0.3;
real k = 0.5;
real x_min = 1;
real x_max = 0;

real f_raw(real x) {return -(x-a)*(x-b)**2;}
real f_min = f_raw(x_min);
real f_max = f_raw(x_max);
real f(real x) {return (f_raw(x)-f_min)/(f_max-f_min);}

// df/dx = -(x-b)**2 - 2(x-a)(x-b)
// at the extremes, df/dx = 0, so
//   (x-b)**2 = -2(x-a)(x-b)
//   so
//        x-b = 0
//          x = b
//   or
//    -2(x-a) = (x-b)
//          x = (2a + b)/3

real x_fold = (2*a+b)/3;
real x_barrier = b;

real const_force(real x) {return f(x_fold)-F*(x-x_fold);}
real force(real x) {return const_force(x)+1/2*k*(x-x_fold)**2;}
real f_force(real x) {return f(x)+(force(x)-f(x_fold));}

draw(graph(f, 0, 1, operator ..), blue);
draw(graph(const_force, 0, 1, operator ..), dashed);
draw(graph(force, 0, 1, operator ..), black);
draw(graph(f_force, 0, 1, operator ..), red);

dot(sLabel("folded",align=S),(x_fold, f(x_fold)));
dot(sLabel("barrier",align=N),(x_barrier, f(x_barrier)));

xlimits(0, 1, Crop);
ylimits(0, 1, Crop);

label(sLabel("Energy landscape schematic"), point(N), N);
xaxis(sLabel("End-to-end distance $x$"), BottomTop);
yaxis(sLabel("Free energy $E$"), LeftRight);
